==> mlr_2.4.tar.gz: mlr/R/aggregations.R <==
#' @title Aggregation methods.
#'
#' @description
#' \itemize{
#'   \item{\bold{test.mean}}{\cr Mean of performance values on test sets.}
#'   \item{\bold{test.sd}}{\cr Standard deviation of performance values on test sets.}
#'   \item{\bold{test.median}}{\cr Median of performance values on test sets.}
#'   \item{\bold{test.min}}{\cr Minimum of performance values on test sets.}
#'   \item{\bold{test.max}}{\cr Maximum of performance values on test sets.}
#'   \item{\bold{test.sum}}{\cr Sum of performance values on test sets.}
#'   \item{\bold{train.mean}}{\cr Mean of performance values on training sets.}
#'   \item{\bold{train.sd}}{\cr Standard deviation of performance values on training sets.}
#'   \item{\bold{train.median}}{\cr Median of performance values on training sets.}
#'   \item{\bold{train.min}}{\cr Minimum of performance values on training sets.}
#'   \item{\bold{train.max}}{\cr Maximum of performance values on training sets.}
#'   \item{\bold{train.sum}}{\cr Sum of performance values on training sets.}
#'   \item{\bold{b632}}{\cr Aggregation for B632 bootstrap.}
#'   \item{\bold{b632plus}}{\cr Aggregation for B632+ bootstrap.}
#'   \item{\bold{testgroup.mean}}{\cr Performance values on test sets are grouped according
#'     to resampling method. The mean for very group is calculated, then the mean of those means.
#'     Mainly used for repeated CV.}
#'   \item{\bold{test.join}}{\cr Performance measure on joined test sets.
#'     This is especially useful for small sample sizes where unbalanced group sizes have a significant impact
#'     on the aggregation, especially for cross-validation test.join might make sense now.
#'     For the repeated CV, the performance is calculated on each repetition and then aggregated
#'     with the arithmetic mean.}
#' }
#' @format None
#' @seealso \code{\link{Aggregation}}
#' @name aggregations
#' @rdname aggregations
NULL

#' @export
#' @rdname aggregations
test.mean = makeAggregation(
  id = "test.mean",
  fun = function(task, perf.test, perf.train, measure, group, pred) mean(perf.test)
)

#' @export
#' @rdname aggregations
test.sd = makeAggregation(
  id = "test.sd",
  fun = function(task, perf.test, perf.train, measure, group, pred) sd(perf.test)
)

#' @export
#' @rdname aggregations
test.median = makeAggregation(
  id = "test.median",
  fun = function(task, perf.test, perf.train, measure, group, pred) median(perf.test)
)

#' @export
#' @rdname aggregations
test.min = makeAggregation(
  id = "test.min",
  fun = function(task, perf.test, perf.train, measure, group, pred) min(perf.test)
)

#' @export
#' @rdname aggregations
test.max = makeAggregation(
  id = "test.max",
  fun = function(task, perf.test, perf.train, measure, group, pred) max(perf.test)
)

#' @export
#' @rdname aggregations
test.sum = makeAggregation(
  id = "test.sum",
  fun = function(task, perf.test, perf.train, measure, group, pred) sum(perf.test)
)

#' @export
#' @rdname aggregations
test.range = makeAggregation(
  id = "test.range",
  fun = function(task, perf.test, perf.train, measure, group, pred) diff(range(perf.test))
)

#' @export
#' @rdname aggregations
test.sqrt.of.mean = makeAggregation(
  id = "test.sqrt.of.mean",
  fun = function(task, perf.test, perf.train, measure, group, pred) sqrt(mean(perf.test))
)

#' @export
#' @rdname aggregations
train.mean = makeAggregation(
  id = "train.mean",
  fun = function(task, perf.test, perf.train, measure, group, pred) mean(perf.train)
)

#' @export
#' @rdname aggregations
train.sd = makeAggregation(
  id = "train.sd",
  fun = function(task, perf.test, perf.train, measure, group, pred) sd(perf.train)
)

#' @export
#' @rdname aggregations
train.median = makeAggregation(
  id = "train.median",
  fun = function(task, perf.test, perf.train, measure, group, pred) median(perf.train)
)

#' @export
#' @rdname aggregations
train.min = makeAggregation(
  id = "train.min",
  fun = function(task, perf.test, perf.train, measure, group, pred) min(perf.train)
)

#' @export
#' @rdname aggregations
train.max = makeAggregation(
  id = "train.max",
  fun = function(task, perf.test, perf.train, measure, group, pred) max(perf.train)
)

#' @export
#' @rdname aggregations
train.sum = makeAggregation(
  id = "train.sum",
  fun = function(task, perf.test, perf.train, measure, group, pred) sum(perf.train)
)

#' @export
#' @rdname aggregations
train.range = makeAggregation(
  id = "train.range",
  fun = function(task, perf.test, perf.train, measure, group, pred) diff(range(perf.train))
)

#' @export
#' @rdname aggregations
train.sqrt.of.mean = makeAggregation(
  id = "train.sqrt.of.mean",
  fun = function(task, perf.test, perf.train, measure, group, pred) sqrt(mean(perf.train))
)

#' @export
#' @rdname aggregations
b632 = makeAggregation(
  id = "b632",
  fun = function(task, perf.test, perf.train, measure, group, pred) {
    mean(0.632*perf.test + 0.368*perf.train)
  }
)


#FIXME: read this again properly and double check it
#' @export
#' @rdname aggregations
b632plus = makeAggregation(
  id = "b632plus",
  fun = function(task, perf.test, perf.train, measure, group, pred) {
    df = as.data.frame(pred)
    a = numeric(length(perf.test))
    for (i in seq_along(a)) {
      df2 = df[df$iter == i,, drop = FALSE]
      y1 = df2$truth
      y2 = df2$response
      grid = expand.grid(y1, y2, KEEP.OUT.ATTRS = FALSE)
      pred2 = makePrediction(task.desc = pred$task.desc, row.names = rownames(grid),
        id = NULL, truth = grid[, 1L], predict.type = "response", y = grid[, 2L],
        time = NA_real_)
      gamma = performance(pred2, measures = measure)
      R = (perf.test[i] - perf.train[i]) / (gamma - perf.train[i])
      w = 0.632 / (1 - 0.368*R)
      a[i] = (1-w) * perf.train[i] + w*perf.test[i]
    }
    return(mean(a))
  }
)

#' @export
#' @rdname aggregations
testgroup.mean = makeAggregation(
  id = "testgroup.mean",
  fun = function(task, perf.test, perf.train, measure, group, pred) {
    mean(vnapply(split(perf.test, group), mean))
  }
)

#' @export
#' @rdname aggregations
test.join = makeAggregation(
  id = "test.join",
  fun = function(task, perf.test, perf.train, measure, group, pred) {
    df = as.data.frame(pred)
    f = if (length(group)) group[df$iter] else factor(rep(1L, nrow(df)))
    mean(vnapply(split(df, f), function(df) {
      npred = makePrediction(task.desc = pred$task.desc, row.names = rownames(df),
        id = NULL, truth = df$truth, predict.type = pred$predict.type, y = df$response,
        time = NA_real_)
      performance(npred, measure)
    }))
  }
)

==> mlr_2.7.tar.gz: mlr/R/aggregations.R <==
#' @title Aggregation methods.
#'
#' @description
#' \itemize{
#'   \item{\bold{test.mean}}{\cr Mean of performance values on test sets.}
#'   \item{\bold{test.sd}}{\cr Standard deviation of performance values on test sets.}
#'   \item{\bold{test.median}}{\cr Median of performance values on test sets.}
#'   \item{\bold{test.min}}{\cr Minimum of performance values on test sets.}
#'   \item{\bold{test.max}}{\cr Maximum of performance values on test sets.}
#'   \item{\bold{test.sum}}{\cr Sum of performance values on test sets.}
#'   \item{\bold{train.mean}}{\cr Mean of performance values on training sets.}
#'   \item{\bold{train.sd}}{\cr Standard deviation of performance values on training sets.}
#'   \item{\bold{train.median}}{\cr Median of performance values on training sets.}
#'   \item{\bold{train.min}}{\cr Minimum of performance values on training sets.}
#'   \item{\bold{train.max}}{\cr Maximum of performance values on training sets.}
#'   \item{\bold{train.sum}}{\cr Sum of performance values on training sets.}
#'   \item{\bold{b632}}{\cr Aggregation for B632 bootstrap.}
#'   \item{\bold{b632plus}}{\cr Aggregation for B632+ bootstrap.}
#'   \item{\bold{testgroup.mean}}{\cr Performance values on test sets are grouped according
#'     to resampling method. The mean for very group is calculated, then the mean of those means.
#'     Mainly used for repeated CV.}
#'   \item{\bold{test.join}}{\cr Performance measure on joined test sets.
#'     This is especially useful for small sample sizes where unbalanced group sizes have a significant impact
#'     on the aggregation, especially for cross-validation test.join might make sense now.
#'     For the repeated CV, the performance is calculated on each repetition and then aggregated
#'     with the arithmetic mean.}
#' }
#' @format None
#' @seealso \code{\link{Aggregation}}
#' @name aggregations
#' @rdname aggregations
NULL

#' @export
#' @rdname aggregations
test.mean = makeAggregation(
  id = "test.mean",
  name = "Test mean",
  fun = function(task, perf.test, perf.train, measure, group, pred) mean(perf.test)
)

#' @export
#' @rdname aggregations
test.sd = makeAggregation(
  id = "test.sd",
  name = "Test sd",
  fun = function(task, perf.test, perf.train, measure, group, pred) sd(perf.test)
)

#' @export
#' @rdname aggregations
test.median = makeAggregation(
  id = "test.median",
  name = "Test median",
  fun = function(task, perf.test, perf.train, measure, group, pred) median(perf.test)
)

#' @export
#' @rdname aggregations
test.min = makeAggregation(
  id = "test.min",
  name = "Test minimum",
  fun = function(task, perf.test, perf.train, measure, group, pred) min(perf.test)
)

#' @export
#' @rdname aggregations
test.max = makeAggregation(
  id = "test.max",
  name = "Test maximum",
  fun = function(task, perf.test, perf.train, measure, group, pred) max(perf.test)
)

#' @export
#' @rdname aggregations
test.sum = makeAggregation(
  id = "test.sum",
  name = "Test sum",
  fun = function(task, perf.test, perf.train, measure, group, pred) sum(perf.test)
)

#' @export
#' @rdname aggregations
test.range = makeAggregation(
  id = "test.range",
  name = "Test range",
  fun = function(task, perf.test, perf.train, measure, group, pred) diff(range(perf.test))
)

#' @export
#' @rdname aggregations
test.rmse = makeAggregation(
  id = "test.rmse",
  name = "Test RMSE",
  fun = function(task, perf.test, perf.train, measure, group, pred) sqrt(mean(perf.test^2))
)

#' @export
#' @rdname aggregations
train.mean = makeAggregation(
  id = "train.mean",
  name = "Training mean",
  fun = function(task, perf.test, perf.train, measure, group, pred) mean(perf.train)
)

#' @export
#' @rdname aggregations
train.sd = makeAggregation(
  id = "train.sd",
  name = "Training sd",
  fun = function(task, perf.test, perf.train, measure, group, pred) sd(perf.train)
)

#' @export
#' @rdname aggregations
train.median = makeAggregation(
  id = "train.median",
  name = "Training median",
  fun = function(task, perf.test, perf.train, measure, group, pred) median(perf.train)
)

#' @export
#' @rdname aggregations
train.min = makeAggregation(
  id = "train.min",
  name = "Training min",
  fun = function(task, perf.test, perf.train, measure, group, pred) min(perf.train)
)

#' @export
#' @rdname aggregations
train.max = makeAggregation(
  id = "train.max",
  name = "Training max",
  fun = function(task, perf.test, perf.train, measure, group, pred) max(perf.train)
)

#' @export
#' @rdname aggregations
train.sum = makeAggregation(
  id = "train.sum",
  name = "Training sum",
  fun = function(task, perf.test, perf.train, measure, group, pred) sum(perf.train)
)

#' @export
#' @rdname aggregations
train.range = makeAggregation(
  id = "train.range",
  name = "Training range",
  fun = function(task, perf.test, perf.train, measure, group, pred) diff(range(perf.train))
)

#' @export
#' @rdname aggregations
train.rmse = makeAggregation(
  id = "train.rmse",
  name = "Training RMSE",
  fun = function(task, perf.test, perf.train, measure, group, pred) sqrt(mean(perf.train^2))
)

#' @export
#' @rdname aggregations
b632 = makeAggregation(
  id = "b632",
  name = ".632 Bootstrap",
  fun = function(task, perf.test, perf.train, measure, group, pred) {
    mean(0.632*perf.test + 0.368*perf.train)
  }
)


#FIXME: read this again properly and double check it
#' @export
#' @rdname aggregations
b632plus = makeAggregation(
  id = "b632plus",
  name = ".632 Bootstrap plus",
  fun = function(task, perf.test, perf.train, measure, group, pred) {
    df = as.data.frame(pred)
    a = numeric(length(perf.test))
    for (i in seq_along(a)) {
      df2 = df[df$iter == i,, drop = FALSE]
      y1 = df2$truth
      y2 = df2$response
      grid = expand.grid(y1, y2, KEEP.OUT.ATTRS = FALSE)
      pred2 = makePrediction(task.desc = pred$task.desc, row.names = rownames(grid),
        id = NULL, truth = grid[, 1L], predict.type = "response", y = grid[, 2L],
        time = NA_real_)
      gamma = performance(pred2, measures = measure)
      R = (perf.test[i] - perf.train[i]) / (gamma - perf.train[i])
      w = 0.632 / (1 - 0.368*R)
      a[i] = (1-w) * perf.train[i] + w*perf.test[i]
    }
    return(mean(a))
  }
)

#' @export
#' @rdname aggregations
testgroup.mean = makeAggregation(
  id = "testgroup.mean",
  name = "Test group mean",
  fun = function(task, perf.test, perf.train, measure, group, pred) {
    mean(vnapply(split(perf.test, group), mean))
  }
)

#' @export
#' @rdname aggregations
test.join = makeAggregation(
  id = "test.join",
  name = "Test join",
  fun = function(task, perf.test, perf.train, measure, group, pred) {
    df = as.data.frame(pred)
    f = if (length(group)) group[df$iter] else factor(rep(1L, nrow(df)))
    mean(vnapply(split(df, f), function(df) {
      npred = makePrediction(task.desc = pred$task.desc, row.names = rownames(df),
        id = NULL, truth = df$truth, predict.type = pred$predict.type, y = df$response,
        time = NA_real_)
      performance(npred, measure)
    }))
  }
)
